//$HeadURL: svn+ssh://rbezema@svn.wald.intevation.org/deegree/base/branches/2.2_testing/src/org/deegree/model/feature/schema/FeatureType.java $
/*----------------    FILE HEADER  ------------------------------------------

 This file is part of deegree.
 Copyright (C) 2001-2008 by:
 EXSE, Department of Geography, University of Bonn
 http://www.giub.uni-bonn.de/deegree/
 lat/lon GmbH
 http://www.lat-lon.de

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 Contact:

 Andreas Poth
 lat/lon GmbH
 Aennchenstr. 19
 53115 Bonn
 Germany
 E-Mail: poth@lat-lon.de

 Prof. Dr. Klaus Greve
 Department of Geography
 University of Bonn
 Meckenheimer Allee 166
 53115 Bonn
 Germany
 E-Mail: greve@giub.uni-bonn.de

 
 ---------------------------------------------------------------------------*/

package org.deegree.model.feature.schema;

import java.net.URI;

import org.deegree.datatypes.QualifiedName;

/**
 * The FeatureType interface is intended to provide details of the type of a Feature that are
 * described as Feature Schema in the Abstract Specification's Essential Model, specifically the
 * names and types of the properties associated with each instance of a Feature of the given
 * FeatureType.
 * <p>
 * -----------------------------------------------------------------------
 * </p>
 * 
 * @author <a href="mailto:poth@lat-lon.de">Andreas Poth</a>
 * @version $Revision: 9343 $ $Date: 2007-12-27 14:30:32 +0100 (Do, 27 Dez 2007) $
 */
public interface FeatureType {

    /**
     * returns the name of the FeatureType
     * 
     * @return name of the FeatureType
     */
    public QualifiedName getName();

    /**
     * Returns whether this feature type is abstract or not.
     * 
     * @return true, if the feature type is abstract, false otherwise
     */
    public boolean isAbstract();

    /**
     * returns the namespace of the feature type (maybe null)
     * 
     * @return the namespace of the feature type (maybe <code>null</code>)
     */
    public URI getNameSpace();

    /**
     * returns the location of the XML schema defintion assigned to a namespace
     * 
     * @return the location of the XML schema defintion assigned to a namespace
     */
    public URI getSchemaLocation();

    /**
     * returns the name of the property a the passed index position
     */
    public QualifiedName getPropertyName( int index );

    /**
     * returns the properties of this feature type
     * 
     * @return type properties
     */
    public PropertyType[] getProperties();

    /**
     * returns a property of this feature type identified by its name
     * 
     * @param name
     *            name of the desired property
     * @return one named property
     */
    public PropertyType getProperty( QualifiedName name );

    /**
     * returns the FeatureTypeProperties of type GEOMETRY
     * 
     * @see org.deegree.datatypes.Types
     * @return the FeatureTypeProperties of type GEOMETRY
     */
    public GeometryPropertyType[] getGeometryProperties();

    /**
     * returns true if the passed FeatureType equals this FeatureType. Two FeatureTypes are equal if
     * they have the same qualified name
     * 
     * @return <code>true</code> if the passed FeatureType equals this FeatureType.
     */
    public boolean equals( FeatureType featureType );

}
