//$HeadURL: $
/*----------------    FILE HEADER  ------------------------------------------
 This file is part of deegree.
 Copyright (C) 2001-2008 by:
 Department of Geography, University of Bonn
 http://www.giub.uni-bonn.de/deegree/
 lat/lon GmbH
 http://www.lat-lon.de

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 Lesser General Public License for more details.
 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 Contact:

 Andreas Poth
 lat/lon GmbH
 Aennchenstr. 19
 53177 Bonn
 Germany
 E-Mail: poth@lat-lon.de

 Prof. Dr. Klaus Greve
 Department of Geography
 University of Bonn
 Meckenheimer Allee 166
 53115 Bonn
 Germany
 E-Mail: greve@giub.uni-bonn.de
 ---------------------------------------------------------------------------*/

package org.deegree.ogcwebservices.wcts;

import org.deegree.framework.log.ILogger;
import org.deegree.framework.log.LoggerFactory;
import org.deegree.i18n.Messages;
import org.deegree.ogcbase.ExceptionCode;
import org.deegree.ogcwebservices.OGCWebService;
import org.deegree.ogcwebservices.OGCWebServiceException;
import org.deegree.ogcwebservices.OGCWebServiceRequest;
import org.deegree.ogcwebservices.getcapabilities.OGCCapabilities;
import org.deegree.ogcwebservices.wcts.configuration.WCTSConfiguration;
import org.deegree.ogcwebservices.wcts.operation.GetResourceByID;
import org.deegree.ogcwebservices.wcts.operation.IsTransformable;
import org.deegree.ogcwebservices.wcts.operation.OperationParameter;
import org.deegree.ogcwebservices.wcts.operation.WCTSGetCapabilities;
import org.deegree.ogcwebservices.wcts.operation.WCTSTransform;

/**
 * The <code>WCTService</code> class TODO add documentation here
 * 
 * @author <a href="mailto:bezema@lat-lon.de">Rutger Bezema</a>
 * 
 * @author last edited by: $Author:$
 * 
 * @version $Revision:$, $Date:$
 * 
 */
public class WCTService implements OGCWebService {
    private static ILogger LOG = LoggerFactory.getLogger( WCTService.class );

    private final WCTSConfiguration config;
    /**
     * The version of this wcts, will be read from the configuration.
     */
    public static String version = "0.4.0";

    /**
     * @param config
     */
    public WCTService( WCTSConfiguration config ) {
        this.config = config;
        synchronized ( LOG ) {
            version = config.getVersion();
            LOG.notifyAll();
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.deegree.ogcwebservices.OGCWebService#doService(org.deegree.ogcwebservices.OGCWebServiceRequest)
     */
    public Object doService( OGCWebServiceRequest request )
                                                           throws OGCWebServiceException {
        Object result = null;
        if ( request != null ) {
            long time = System.currentTimeMillis();
            LOG.logDebug( "Incoming request with id: " + request.getId() );
            if ( request instanceof GetResourceByID ) {
                // handle transformable.
            } else if ( request instanceof IsTransformable ) {
                // handle IsTransformable.
            } else if ( request instanceof OperationParameter ) {
                // handle transformable.
            } else if ( request instanceof WCTSGetCapabilities ) {
                result = getCapabilities();
            } else if ( request instanceof WCTSTransform ) {
                // handle transformable.
            } else {
                throw new OGCWebServiceException( request.toString(),
                                                  Messages.getMessage( "WCTS_UNKNOWN_REQUEST" ),
                                                  ExceptionCode.OPERATIONNOTSUPPORTED );
            }
            LOG.logDebug( "The handling of request with id: " + request.getId()
                          + " took: "
                          + ( System.currentTimeMillis() - time )
                          / 1000.
                          + " seconds" );
        }

        return result;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.deegree.ogcwebservices.OGCWebService#getCapabilities()
     */
    public OGCCapabilities getCapabilities() {
        return config;
    }

}
