//$HeadURL: $
/*----------------    FILE HEADER  ------------------------------------------
 This file is part of deegree.
 Copyright (C) 2001-2008 by:
 Department of Geography, University of Bonn
 http://www.giub.uni-bonn.de/deegree/
 lat/lon GmbH
 http://www.lat-lon.de

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 Lesser General Public License for more details.
 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 Contact:

 Andreas Poth
 lat/lon GmbH
 Aennchenstr. 19
 53177 Bonn
 Germany
 E-Mail: poth@lat-lon.de

 Prof. Dr. Klaus Greve
 Department of Geography
 University of Bonn
 Meckenheimer Allee 166
 53115 Bonn
 Germany
 E-Mail: greve@giub.uni-bonn.de
 ---------------------------------------------------------------------------*/

package org.deegree.ogcwebservices.wcts;

import java.io.IOException;
import java.net.URL;

import org.deegree.framework.log.ILogger;
import org.deegree.framework.log.LoggerFactory;
import org.deegree.ogcbase.ExceptionCode;
import org.deegree.ogcwebservices.OGCWebServiceException;
import org.deegree.ogcwebservices.getcapabilities.InvalidCapabilitiesException;
import org.deegree.ogcwebservices.wcts.configuration.WCTSConfiguration;
import org.deegree.ogcwebservices.wcts.configuration.WCTSConfigurationDocument;
import org.xml.sax.SAXException;

/**
 * <code>WCTServiceFactory</code> a convenience class to create and receive a {@link WCTSConfiguration} instance.
 * 
 * @author <a href="mailto:bezema@lat-lon.de">Rutger Bezema</a>
 * 
 * @author last edited by: $Author:$
 * 
 * @version $Revision:$, $Date:$
 * 
 */
public class WCTServiceFactory {
    private static ILogger LOG = LoggerFactory.getLogger( WCTServiceFactory.class );

    private static WCTSConfiguration CONFIG;

    private WCTServiceFactory() {
        // do nottin
    }

    /**
     * @param configURL
     *            to read the configuration from.
     */
    public static synchronized void setConfiguration( URL configURL ) {
        if ( configURL != null ) {
            try {
                WCTSConfigurationDocument doc = new WCTSConfigurationDocument( configURL );
                CONFIG = doc.parseConfiguration();
            } catch ( IOException e ) {
                LOG.logError( e.getMessage(), e );
            } catch ( SAXException e ) {
                LOG.logError( e.getMessage(), e );
            } catch ( InvalidCapabilitiesException e ) {
                LOG.logError( e.getMessage(), e );
            }
        }
    }

    /**
     * @return the configuration
     * @throws OGCWebServiceException
     *             if no configuration has been created yet.
     */
    public static synchronized WCTSConfiguration getConfiguration()
                                                                   throws OGCWebServiceException {
        if ( CONFIG == null ) {
            throw new OGCWebServiceException( "Configuration has not been initialized yet",
                                              ExceptionCode.INTERNAL_SERVER_ERROR );
        }
        return CONFIG;
    }

    /**
     * 
     * @param wfsConfiguration
     * @return
     * @throws OGCWebServiceException
     */
    public static synchronized WCTService createServiceInstance()
                                                                 throws OGCWebServiceException {
        if ( CONFIG == null ) {
            throw new OGCWebServiceException( "Configuration has not been initialized yet",
                                              ExceptionCode.INTERNAL_SERVER_ERROR );
        }
        return new WCTService( CONFIG );
    }

}
