//$HeadURL: $
/*----------------    FILE HEADER  ------------------------------------------
 This file is part of deegree.
 Copyright (C) 2001-2008 by:
 Department of Geography, University of Bonn
 http://www.giub.uni-bonn.de/deegree/
 lat/lon GmbH
 http://www.lat-lon.de

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 Lesser General Public License for more details.
 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 Contact:

 Andreas Poth
 lat/lon GmbH
 Aennchenstr. 19
 53177 Bonn
 Germany
 E-Mail: poth@lat-lon.de

 Prof. Dr. Klaus Greve
 Department of Geography
 University of Bonn
 Meckenheimer Allee 166
 53115 Bonn
 Germany
 E-Mail: greve@giub.uni-bonn.de
 ---------------------------------------------------------------------------*/

package org.deegree.ogcwebservices.wcts.operation;

import java.util.ArrayList;
import java.util.List;

import org.deegree.ogcwebservices.AbstractOGCWebServiceRequest;

/**
 * <code>GetResourceByID</code> encapsulates the xml-dom representation or kvp request parameters of a GetResourceById
 * request.
 * 
 * @author <a href="mailto:bezema@lat-lon.de">Rutger Bezema</a>
 * 
 * @author last edited by: $Author:$
 * 
 * @version $Revision:$, $Date:$
 * 
 */
public class GetResourceByID extends AbstractOGCWebServiceRequest {

    private static final long serialVersionUID = 9201043457053591617L;

    private final List<String> resourceIDs;

    private final String outputFormat;

    /**
     * @param id
     *            of the request
     * @param version
     *            of the request
     * @param resourceIDs
     * @param outputFormat
     */
    public GetResourceByID( String id, String version, List<String> resourceIDs, String outputFormat ) {
        super( version, id, null );
        if( resourceIDs == null ){
            resourceIDs = new ArrayList<String>();
        }
        this.resourceIDs = resourceIDs;

        
        if( outputFormat== null ){
            outputFormat = new String();
        }
        this.outputFormat = outputFormat;

    }

    /**
     * @return the resourceIDs, may be empty but never <code>null</code>
     */
    public final List<String> getResourceIDs() {
        return resourceIDs;
    }

    /**
     * @return the outputFormat, may be empty but never <code>null</code>
     */
    public final String getOutputFormat() {
        return outputFormat;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.deegree.ogcwebservices.OGCWebServiceRequest#getServiceName()
     */
    public String getServiceName() {
        return "WCTS";
    }

}
