//$HeadURL: svn+ssh://rbezema@svn.wald.intevation.org/deegree/base/branches/2.2_testing/src/org/deegree/ogcwebservices/wpvs/configuration/RemoteWFSDataSource.java $
/*----------------    FILE HEADER  ------------------------------------------

 This file is part of deegree.
 Copyright (C) 2001-2008 by:
 EXSE, Department of Geography, University of Bonn
 http://www.giub.uni-bonn.de/deegree/
 lat/lon GmbH
 http://www.lat-lon.de

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 Contact:

 Andreas Poth
 lat/lon GmbH
 Aennchenstraße 19
 53177 Bonn
 Germany
 E-Mail: poth@lat-lon.de

 Prof. Dr. Klaus Greve
 Department of Geography
 University of Bonn
 Meckenheimer Allee 166
 53115 Bonn
 Germany
 E-Mail: greve@giub.uni-bonn.de
 
 ---------------------------------------------------------------------------*/

package org.deegree.ogcwebservices.wpvs.configuration;

import java.io.IOException;
import java.net.URL;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import org.deegree.datatypes.QualifiedName;
import org.deegree.i18n.Messages;
import org.deegree.model.filterencoding.Filter;
import org.deegree.model.spatialschema.Surface;
import org.deegree.ogcbase.PropertyPath;
import org.deegree.ogcwebservices.OGCWebService;
import org.deegree.ogcwebservices.OGCWebServiceException;
import org.deegree.ogcwebservices.wfs.RemoteWFService;
import org.deegree.ogcwebservices.wfs.capabilities.WFSCapabilities;
import org.deegree.ogcwebservices.wfs.capabilities.WFSCapabilitiesDocument;
import org.deegree.ogcwebservices.wpvs.capabilities.OWSCapabilities;
import org.xml.sax.SAXException;

/**
 * This class represents a remote WFS dataSource object.
 * 
 * @author <a href="mailto:mays@lat-lon.de">Judit Mays</a>
 * @author last edited by: $Author: apoth $
 * 
 * $Revision: 9345 $, $Date: 2007-12-27 17:22:25 +0100 (Do, 27 Dez 2007) $
 * 
 */
public class RemoteWFSDataSource extends LocalWFSDataSource {

    // private static final ILogger LOG = LoggerFactory.getLogger( RemoteWFSDataSource.class );
    private static Map<URL, WFSCapabilities> cache = new ConcurrentHashMap<URL, WFSCapabilities>();

    /**
     * Creates a new <code>RemoteWFSDataSource</code> object from the given parameters.
     * 
     * @param name
     * @param owsCapabilities
     * @param validArea
     * @param minScaleDenominator
     * @param maxScaleDenominator
     * @param geometryProperty
     * @param filterCondition
     * @param maxFeatures 
     */
    public RemoteWFSDataSource( QualifiedName name, OWSCapabilities owsCapabilities,
                                Surface validArea, double minScaleDenominator,
                                double maxScaleDenominator, PropertyPath geometryProperty,
                                Filter filterCondition, int maxFeatures) {

        super( name, owsCapabilities, validArea, minScaleDenominator, maxScaleDenominator,
               geometryProperty, filterCondition,  maxFeatures );
        setServiceType( REMOTE_WFS );

    }

    @Override
    public OGCWebService getOGCWebService()
                            throws OGCWebServiceException {
        WFSCapabilities wfsCapabilities = null;
        synchronized ( this ) {
            URL url = getOwsCapabilities().getOnlineResource();
            wfsCapabilities = cache.get( url );
            if ( !cache.containsKey( url ) || wfsCapabilities == null ) {
                WFSCapabilitiesDocument wfsCapsDoc = new WFSCapabilitiesDocument();
                try {
                    wfsCapsDoc.load( url );
                    wfsCapabilities = (WFSCapabilities) wfsCapsDoc.parseCapabilities();
                    cache.put( url, wfsCapabilities );
                } catch ( IOException e ) {
                    throw new OGCWebServiceException(
                                                      Messages.getMessage(
                                                                           "WPVS_DATASOURCE_CAP_ERROR",
                                                                           toString() )
                                                                              + e.getMessage() );
                } catch ( SAXException e ) {
                    throw new OGCWebServiceException(
                                                      Messages.getMessage(
                                                                           "WPVS_DATASOURCE_CAP_ERROR",
                                                                           toString() )
                                                                              + e.getMessage() );
                }
            }

        }
        return new RemoteWFService( wfsCapabilities );
    }

}