//$HeadURL: svn+ssh://rbezema@svn.wald.intevation.org/deegree/base/branches/2.2_testing/src/org/deegree/ogcwebservices/wpvs/j3d/TexturedSurface.java $
/*----------------    FILE HEADER  ------------------------------------------

 This file is part of deegree.
 Copyright (C) 2001-2008 by:
 EXSE, Department of Geography, University of Bonn
 http://www.giub.uni-bonn.de/deegree/
 lat/lon GmbH
 http://www.lat-lon.de

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 Contact:

 Andreas Poth
 lat/lon GmbH
 Aennchenstr. 19
 53177 Bonn
 Germany
 E-Mail: poth@lat-lon.de

 Prof. Dr. Klaus Greve
 Department of Geography
 University of Bonn
 Meckenheimer Allee 166
 53115 Bonn
 Germany
 E-Mail: greve@giub.uni-bonn.de

 ---------------------------------------------------------------------------*/
package org.deegree.ogcwebservices.wpvs.j3d;

import java.awt.image.BufferedImage;
import java.io.IOException;
import java.net.URL;

import javax.media.j3d.Appearance;
import javax.media.j3d.Material;
import javax.media.j3d.TexCoordGeneration;
import javax.media.j3d.Texture;
import javax.vecmath.Vector4f;

import org.deegree.framework.util.ImageUtils;
import org.deegree.model.spatialschema.Position;
import org.deegree.model.spatialschema.Ring;
import org.deegree.model.spatialschema.Surface;
import org.deegree.ogcwebservices.wpvs.configuration.RenderingConfiguration;

import com.sun.j3d.utils.geometry.GeometryInfo;
import com.sun.j3d.utils.geometry.NormalGenerator;
import com.sun.j3d.utils.image.TextureLoader;

/**
 * 
 * 
 * 
 * @version $Revision: 9345 $
 * @author <a href="mailto:poth@lat-lon.de">Andreas Poth</a>
 * @author last edited by: $Author: apoth $
 * 
 * @version 1.0. $Revision: 9345 $, $Date: 2007-12-27 17:22:25 +0100 (Do, 27 Dez 2007) $
 * 
 * @since 2.0
 */
public class TexturedSurface extends ColoredSurface {

    private Texture texture = null;

    private BufferedImage textureImg = null;

    private float[][] textureCoords = null;
    
    /**
     * creates a TexturedSurface from a geometry, color informations and a texture image. Since a
     * texture image be somehow transparent it is useful to be able to define a surfaces color. This
     * constructor will use default coordinates to adjust a texture onto a surface.
     * 
     * @param objectID
     *            an Id for this Surface, for example a db primary key
     * @param parentID
     *            an Id for the parent of this Surface, for example if this is a wall the parent is
     *            the building.
     * @param surface
     *            the ogc:geometry surface which holds the point references of a polygon, not to be
     *            confused with a j3d Object which this class represents.
     * @param red
     * @param green
     * @param blue
     * @param transparency
     * @param textureImg
     */
    public TexturedSurface( String objectID, String parentID, Surface surface, float red,
                           float green, float blue, float transparency, BufferedImage textureImg ) {
        super( objectID, parentID, surface, red, green, blue, transparency );
        this.textureImg = textureImg;
        createTexture( textureImg );
        setAppearance( createAppearance() );
    }

    /**
     * creates a TexturedSurface from a geometry, Material and a texture image. Since a texture
     * image be somehow transparent it is useful to be able to define a surfaces color. This
     * constructor will use default coordinates to adjust a texture onto a surface.
     * 
     * @param objectID
     *            an Id for this Surface, for example a db primary key
     * @param parentID
     *            an Id for the parent of this Surface, for example if this is a wall the parent is
     *            the building.
     * @param surface
     *            the ogc:geometry surface which holds the point references of a polygon, not to be
     *            confused with a j3d Object which this class represents.
     * @param material
     * @param transparency
     * @param textureImg
     */
    public TexturedSurface( String objectID, String parentID, Surface surface, Material material,
                           float transparency, BufferedImage textureImg ) {
        super( objectID, parentID, surface, material, transparency );
        this.textureImg = textureImg;
        createTexture( textureImg );
        setAppearance( createAppearance() );
    }

    /**
     * creates a TexturedSurface from a geometry, color informations and a reference to a texture
     * image. Since a texture image be somehow transparent it is useful to be able to define a
     * surfaces color. This constructor will use default coordinates to adjust a texture onto a
     * surface.
     * 
     * @param objectID
     *            an Id for this Surface, for example a db primary key
     * @param parentID
     *            an Id for the parent of this Surface, for example if this is a wall the parent is
     *            the building.
     * @param surface
     *            the ogc:geometry surface which holds the point references of a polygon, not to be
     *            confused with a j3d Object which this class represents.
     * @param red
     * @param green
     * @param blue
     * @param transparency
     * @param textureImg
     * @throws IOException
     */
    public TexturedSurface( String objectID, String parentID, Surface surface, float red,
                           float green, float blue, float transparency, URL textureImg )
                            throws IOException {
        super( objectID, parentID, surface, red, green, blue, transparency );
        this.textureImg = ImageUtils.loadImage( textureImg );
        createTexture( this.textureImg );
        setAppearance( createAppearance() );
    }

    /**
     * creates a TexturedSurface from a geometry, Material and a reference to a texture image. Since
     * a texture image be somehow transparent it is useful to be able to define a surfaces color.
     * This constructor will use default coordinates to adjust a texture onto a surface.
     * 
     * @param objectID
     *            an Id for this Surface, for example a db primary key
     * @param parentID
     *            an Id for the parent of this Surface, for example if this is a wall the parent is
     *            the building.
     * @param surface
     *            the ogc:geometry surface which holds the point references of a polygon, not to be
     *            confused with a j3d Object which this class represents.
     * @param material
     * @param transparency
     * @param textureImg
     * @throws IOException
     */
    public TexturedSurface( String objectID, String parentID, Surface surface, Material material,
                           float transparency, URL textureImg ) throws IOException {
        super( objectID, parentID, surface, material, transparency );

        this.textureImg = ImageUtils.loadImage( textureImg );
        createTexture( this.textureImg );
        setAppearance( createAppearance() );
    }

    /**
     * creates a TexturedSurface from a geometry, color informations and a texture image. Since a
     * texture image be somehow transparent it is useful to be able to define a surfaces color.
     * 
     * @param objectID
     *            an Id for this Surface, for example a db primary key
     * @param parentID
     *            an Id for the parent of this Surface, for example if this is a wall the parent is
     *            the building.
     * @param surface
     *            the ogc:geometry surface which holds the point references of a polygon, not to be
     *            confused with a j3d Object which this class represents.
     * @param red
     * @param green
     * @param blue
     * @param transparency
     * @param textureImg
     * @param textureCoords
     */
    public TexturedSurface( String objectID, String parentID, Surface surface, float red,
                           float green, float blue, float transparency, BufferedImage textureImg,
                           float[][] textureCoords ) {
        super( objectID, parentID, surface, red, green, blue, transparency );

        this.textureImg = textureImg;
        this.textureCoords = textureCoords;
        createTexture( textureImg );
        setAppearance( createAppearance() );

    }

    /**
     * creates a TexturedSurface from a geometry, Material and a texture image. Since a texture
     * image be somehow transparent it is useful to be able to define a surfaces color.
     * 
     * @param objectID
     *            an Id for this Surface, for example a db primary key
     * @param parentID
     *            an Id for the parent of this Surface, for example if this is a wall the parent is
     *            the building.
     * @param surface
     *            the ogc:geometry surface which holds the point references of a polygon, not to be
     *            confused with a j3d Object which this class represents.
     * @param material
     * @param transparency
     * @param textureImg
     * @param textureCoords
     */
    public TexturedSurface( String objectID, String parentID, Surface surface, Material material,
                           float transparency, BufferedImage textureImg, float[][] textureCoords ) {
        super( objectID, parentID, surface, material, transparency );

        this.textureImg = textureImg;
        this.textureCoords = textureCoords;
        createTexture( textureImg );
        setAppearance( createAppearance() );
    }

    /**
     * creates a TexturedSurface from a geometry, color informations and a reference to a texture
     * image. Since a texture image be somehow transparent it is useful to be able to define a
     * surfaces color.
     * 
     * @param objectID
     *            an Id for this Surface, for example a db primary key
     * @param parentID
     *            an Id for the parent of this Surface, for example if this is a wall the parent is
     *            the building.
     * @param surface
     *            the ogc:geometry surface which holds the point references of a polygon, not to be
     *            confused with a j3d Object which this class represents.
     * @param red
     * @param green
     * @param blue
     * @param transparency
     * @param textureImg
     * @param textureCoords
     * @throws IOException
     */
    public TexturedSurface( String objectID, String parentID, Surface surface, float red,
                           float green, float blue, float transparency, URL textureImg,
                           float[][] textureCoords ) throws IOException {
        super( objectID, parentID, surface, red, green, blue, transparency );

        this.textureImg = ImageUtils.loadImage( textureImg );
        this.textureCoords = textureCoords;
        createTexture( this.textureImg );
        setAppearance( createAppearance() );
    }

    /**
     * creates a TexturedSurface from a geometry, Material and a reference to a texture image. Since
     * a texture image be somehow transparent it is useful to be able to define a surfaces color.
     * 
     * @param objectID
     *            an Id for this Surface, for example a db primary key
     * @param parentID
     *            an Id for the parent of this Surface, for example if this is a wall the parent is
     *            the building.
     * @param surface
     *            the ogc:geometry surface which holds the point references of a polygon, not to be
     *            confused with a j3d Object which this class represents.
     * @param material
     * @param transparency
     * @param textureImg
     * @param textureCoords
     * @throws IOException
     */
    public TexturedSurface( String objectID, String parentID, Surface surface, Material material,
                           float transparency, URL textureImg, float[][] textureCoords )
                            throws IOException {
        super( objectID, parentID, surface, material, transparency );
        this.textureImg = ImageUtils.loadImage( textureImg );
        this.textureCoords = textureCoords;
        createTexture( this.textureImg );
        setAppearance( createAppearance() );
    }

    private void createTexture( BufferedImage textureImg ) {
        try {
            texture = new TextureLoader( textureImg ).getTexture();
            texture.setEnable( true );
            texture.setCapability( Texture.ALLOW_ENABLE_WRITE );
        } catch ( Exception e ) {
            e.printStackTrace();
        }
    }

    /**
     * @return the texture of this surface.
     */
    public BufferedImage getTexture() {
        return textureImg;
    }

    /**
     * this method must be called before addin the surface to a Group
     */
    @Override
    public void compile() {

        GeometryInfo geometryInfo = new GeometryInfo( GeometryInfo.POLYGON_ARRAY );

        Position[] pos = surface.getSurfaceBoundary().getExteriorRing().getPositions();
        Ring[] innerRings = surface.getSurfaceBoundary().getInteriorRings();
        //the last point is the same as the first, therefore not needed.
        int numberOfPointsInSurface = pos.length -1; 
        int k = 1;
        int l = 3 * (numberOfPointsInSurface );
        if ( innerRings != null ) {
            for ( int i = 0; i < innerRings.length; i++ ) {
                k++;
                l += ( 3 * innerRings[i].getPositions().length );
            }
        }

        float[] coords = new float[l];
        int contourCounts[] = { k };
        int[] stripCounts = new int[k];
        k = 0;
        stripCounts[k++] = numberOfPointsInSurface;
        //System.out.println( "surface length: " + pos.length );
        int z = 0;
        for ( int i = 0; i < numberOfPointsInSurface; i++ ) {
            coords[z++] = (float) pos[i].getX();
            coords[z++] = (float) pos[i].getY();
            coords[z++] = (float) pos[i].getZ();
        }

        if ( innerRings != null ) {
            for ( int j = 0; j < innerRings.length; j++ ) {
                pos = innerRings[j].getPositions();
                stripCounts[k++] = pos.length;
                for ( int i = 0; i < pos.length; i++ ) {
                    coords[z++] = (float) pos[i].getX();
                    coords[z++] = (float) pos[i].getY();
                    coords[z++] = (float) pos[i].getZ();
                }
            }
        }

        geometryInfo.setCoordinates( coords );
        geometryInfo.setStripCounts( stripCounts );
        geometryInfo.setContourCounts( contourCounts );

        // manually set texture coordinates must be done befor
        // normals all calculated
        if ( textureCoords != null ) {
            geometryInfo.setTextureCoordinateParams( textureCoords.length, 2 );
            
            float[] texc = new float[numberOfPointsInSurface*2];
            for ( int i = 0; i < textureCoords.length ; i++ ) {
             for( int newCount = 0; newCount < (numberOfPointsInSurface*2); newCount++ ){
                    texc[newCount] = textureCoords[i][newCount];
                }
                geometryInfo.setTextureCoordinates( i, texc );
            }
        }

        NormalGenerator ng = new NormalGenerator();
        ng.generateNormals( geometryInfo );

        setGeometry( geometryInfo.getGeometryArray() );

        setAppearanceOverrideEnable( true );
    }

    private Appearance createAppearance() {
        Appearance ap = getAppearance();
        if ( texture != null ) {
            ap.setTexture( texture );
            ap.setTextureAttributes( RenderingConfiguration.getInstance().getTextureAttributes() );

            if ( textureCoords == null ) {
                // automatic creation of texture coordinates is behavior of
                // the appearance
                TexCoordGeneration tcg = new TexCoordGeneration(
                                                                 TexCoordGeneration.OBJECT_LINEAR,
                                                                 TexCoordGeneration.TEXTURE_COORDINATE_2 );
                tcg.setPlaneS( new Vector4f( 1, 1, 0, 0 ) );
                tcg.setPlaneT( new Vector4f( 0, 0, 1, 0 ) );
                ap.setTexCoordGeneration( tcg );
            }
        }
        return ap;
    }

}

