//$HeadURL: svn+ssh://rbezema@svn.wald.intevation.org/deegree/base/branches/2.2_testing/src/org/deegree/portal/context/LayerList.java $
/*----------------    FILE HEADER  ------------------------------------------

 This file is part of deegree.
 Copyright (C) 2001-2008 by:
 EXSE, Department of Geography, University of Bonn
 http://www.giub.uni-bonn.de/deegree/
 lat/lon GmbH
 http://www.lat-lon.de

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 Contact:

 Andreas Poth
 lat/lon GmbH
 Aennchenstr. 19
 53115 Bonn
 Germany
 E-Mail: poth@lat-lon.de

 Prof. Dr. Klaus Greve
 Department of Geography
 University of Bonn
 Meckenheimer Allee 166
 53115 Bonn
 Germany
 E-Mail: greve@giub.uni-bonn.de

 
 ---------------------------------------------------------------------------*/
package org.deegree.portal.context;

import java.util.ArrayList;
import java.util.List;

import org.deegree.portal.PortalException;

/**
 * 
 * 
 *
 * @version $Revision: 9346 $
 * @author <a href="mailto:poth@lat-lon.de">Andreas Poth</a>
 * @author last edited by: $Author: apoth $
 *
 * @version 1.0. $Revision: 9346 $, $Date: 2007-12-27 17:39:07 +0100 (Do, 27 Dez 2007) $
 *
 * @since 2.0
 */
public class LayerList {

    private List<Layer> list = new ArrayList<Layer>( 50 );

    private List<Layer> treeList = new ArrayList<Layer>( 50 );

    /**
     * Creates a new LayerList object.
     *
     * @param layers 
     */
    public LayerList( Layer[] layers ) {
        setLayers( layers );
    }

    /**
     * returns a layer identifies by its name
     *
     * @param name name ofthe layer
     *
     * @return names layer
     * @deprecated use @see #getLayer(String, String) instead
     */
    public Layer getLayer( String name ) {
        for ( Layer layer : list ) {
            if ( layer.getName().equals(  name ) ) {
                return layer;
            }
        }
        return null;
    }

    /**
     * returns a layer identifies by its name and service address
     *
     * @param name name ofthe layer
     * @param serverAddress address of the server which servers the layer
     *
     * @return named layer
     */
    public Layer getLayer( String name, String serverAddress ) {
        
        if ( serverAddress != null ) {
            for ( Layer layer : list ) {
                String s = layer.getServer().getOnlineResource().toExternalForm();
                if ( layer.getName().equals( name ) && s.equals( serverAddress ) ) {
                    return layer;
                }
            }
        } else {
            for ( Layer layer : list ) {
                if ( layer.getName().equals(  name ) ) {
                    return layer;
                }
            }
        }
        return null;
    }

    /**
     * returns all layers of the web map context
     *
     * @return array of layers
     */
    public Layer[] getLayers() {
        Layer[] cl = new Layer[list.size()];
        return list.toArray( cl );
    }

    /**
     * sets all layers of the web map context
     *
     * @param layers 
     */
    public void setLayers( Layer[] layers ) {
        this.list.clear();

        if ( layers != null ) {
            for ( int i = 0; i < layers.length; i++ ) {
                list.add( layers[i] );
            }
        }
    }

    /**
     * returns the layers of a node of the tree of the web map context
     *
     * @return array of layers
     */
    public Layer[] getLayersByNodeId( int id ) {
        List<Layer> nodeLayerList = new ArrayList<Layer>( list.size() );
        int parentNodeId;
        for ( int k = 0; k < list.size(); k++ ) {
            parentNodeId = ( list.get( k ) ).getExtension().getParentNodeId();
            if ( parentNodeId == id ) {
                nodeLayerList.add( list.get( k ) );
            }
        }
        Layer[] nodeLayers = new Layer[nodeLayerList.size()];
        return nodeLayerList.toArray( nodeLayers );
    }

    /**
     * TODO: review this changed; it has been introduced as of TreeLayerView Portlet
     * @param root
     */
    public void orderLayerListByLayerTree( Node root ) {
        treeList.clear();
        Node[] rootNode = new Node[1];
        rootNode[0] = root;
        treeList = getLayerListByNode( rootNode );
        setLayers( treeList.toArray( new Layer[treeList.size()] ) );
    }

    /**
     * 
     * @param nodes
     * @return list of layers
     */
    private List<Layer> getLayerListByNode( Node[] nodes ) {
        for ( int i = 0; i < nodes.length; i++ ) {
            getLayerListByNode( nodes[i].getNodes() );
            Layer[] nl = getLayersByNodeId( nodes[i].getId() );
            for ( int j = 0; j < nl.length; j++ ) {
                treeList.add( nl[j] );
            }

        }
        return treeList;
    }

    /**
     * adds one layer to the the web map context. If a layer with the same
     * name as the passed layer already exits it will be overwritten
     *
     * @param layer 
     */
    public void addLayer( Layer layer ) {
        list.add( layer );        
    }

    /**
     * adds one layer to the top of the web map context. If a layer with the same
     * name as the passed layer already exits it will be overwritten
     *
     * @param layer 
     */
    public void addLayerToTop( Layer layer ) {
        list.add( 0, layer );     
    }

    /**
     * removes a layer identified by its name from the web map context
     *
     * @param name name of the layer to be removed
     *
     * @return removed layer
     * @deprecated use @see #removeLayer(String, String) instead
     */
    public Layer removeLayer( String name ) {
        Layer layer = getLayer( name );
        list.remove( layer );
        return layer;
    }
    
    /**
     * removes a layer identified by its name from the web map context
     *
     * @param name name of the layer to be removed
     * @param serverAddress
     *
     * @return removed layer
     */
    public Layer removeLayer( String name, String serverAddress ) {
        Layer layer = getLayer( name, serverAddress );
        list.remove( layer );
        return layer;
    }

    /**
     * moves a layer within the layer list up or down
     * @param layer layer to be moved
     * @param up if true the layer will be moved up otherwise 
     *           it will be moved down
     * @throws PortalException will be thrown if the layer is not known 
     *                         by the layer list.
     */
    public void move( Layer layer, boolean up ) {
        int i = 0;
        Layer target = null;
        while ( i < list.size() && target == null ) {
            Layer tmp = list.get( i );
            if ( tmp.getName().equals( layer.getName() )
                 && tmp.getServer().getOnlineResource().equals(
                                                                layer.getServer().getOnlineResource() ) ) {
                target = tmp;
            }
            i++;
        }
        i--;
        if ( i > 0 && up ) {
            Layer o = list.get( i );
            list.set( i, list.get( i - 1 ) );
            list.set( i - 1, o );
        } else if ( i < list.size() - 1 && !up ) {
            Layer o = list.get( i );
            list.set( i, list.get( i + 1 ) );
            list.set( i + 1, o );
        }
    }

    /**
     * moves a layer within the layer list before the beforeLayer
     * @param layer layer to be moved
     * @param beforeLayer 
     * 
     */
    public void move( Layer layer, Layer beforeLayer ) {
        int i = 0;
        ArrayList<Layer> newList = new ArrayList<Layer>( list.size() );

        while ( i < list.size() ) {
            Layer tmp = list.get( i );
            if ( tmp.getName().equals( beforeLayer.getName() )
                 && tmp.getServer().getOnlineResource().equals(
                                                                beforeLayer.getServer().getOnlineResource() ) ) {
             	newList.add( layer );
                newList.add( beforeLayer );
            } else if ( tmp.getName().equals( layer.getName() )
                        && tmp.getServer().getOnlineResource().equals(
                                                                       layer.getServer().getOnlineResource() ) ) {
            	//do nothing
            } else {
                 newList.add( tmp );
            }
            i++;
        }
        list.clear();
        list = newList;
   }

    /**
     * removes all layers from the web map context
     */
    public void clear() {
        list.clear();
    }

}
