//$HeadURL: svn+ssh://rbezema@svn.wald.intevation.org/deegree/base/branches/2.2_testing/src/org/deegree/portal/context/Node.java $
/*----------------    FILE HEADER  -----------------------------------------

 This file is part of deegree.
 Copyright (C) 2001-2008 by:
 EXSE, Department of Geography, University of Bonn
 http://www.giub.uni-bonn.de/deegree/
 lat/lon GmbH
 http://www.lat-lon.de

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 Contact:

 Andreas Poth
 lat/lon GmbH
 Aennchenstr. 19
 53115 Bonn
 Germany
 E-Mail: poth@lat-lon.de

 Prof. Dr. Klaus Greve
 Department of Geography
 University of Bonn
 Meckenheimer Allee 166
 53115 Bonn
 Germany
 E-Mail: greve@giub.uni-bonn.de

 
 ---------------------------------------------------------------------------*/
package org.deegree.portal.context;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * encapsulates about a node described/contained by a Web Map Context
 * 
 * @version $Revision: 9346 $
 * @author <a href="mailto:vesll@idgis.nl">Linda Vels</a>
 * @author last edited by: $Author: apoth $
 * 
 * @version 1.0. $Revision: 9346 $, $Date: 2007-12-27 17:39:07 +0100 (Do, 27 Dez 2007) $
 * 
 * @since 2.0
 */
public class Node {

    private int id;

    private String title = null;

    private boolean selectable = false;

    private boolean collapsed = false;

    private Node[] nodes = new Node[0];

    private Node parent = null;

    List<Node[]> tree = new ArrayList<Node[]>( 50 );

    /**
     * Creates a new ContextNode object.
     * 
     * @param id
     *            id of the selected node
     * @param parent
     * @param title
     *            title of the selected node
     * @param selectable
     * @param collapsed
     *            defines if the node is collapsed in the legend viewer
     * @throws ContextException
     */
    public Node( int id, Node parent, String title, boolean selectable, boolean collapsed ) throws ContextException {
        setId( id );
        setParent( parent );
        setTitle( title );
        setCollapsed( collapsed );
        setSelectable( selectable );
    }

    /**
     * The childnodes of the selected node in the tree
     * 
     * @return all nodes
     */
    public Node[] getNodes() {
        return nodes;
    }

    /**
     * Returns a childnodes of the selected node by id
     * 
     * @return node by id
     */
    public Node getNode( int nodeId ) {
        Node node = null;
        for ( int i = 0; i < nodes.length; i++ ) {
            node = nodes[i].getNode( nodeId );
            if ( node != null ) {
                return node;
            }
            if ( nodes[i].getId() == nodeId ) {
                return nodes[i];
            }
        }
        return node;
    }

    /**
     * 
     * @param nodeId
     * @param nodes
     * @return node by id from a list
     */
    public Node getNode( int nodeId, Node[] nodes ) {
        Node node = null;
        for ( int i = 0; i < nodes.length; i++ ) {
            node = nodes[i].getNode( nodeId, nodes[i].getNodes() );
            if ( node != null ) {
                return node;
            } else {
                if ( nodes[i].getId() == nodeId ) {
                    return nodes[i];
                }
            }

        }
        return node;
    }

    /**
     * return the maximum id of all nodes
     * 
     * @return maximum id of all nodes
     */
    public int getMaxNodeId() {
        int maxNodeId = id;
        for ( int i = 0; i < nodes.length; i++ ) {
            Node[] brancheNodes = nodes[i].getNodes();
            if ( nodes[i].getId() > maxNodeId ) {
                maxNodeId = nodes[i].getId();

            }
            maxNodeId = getMaxId( brancheNodes, maxNodeId );
        }
        return maxNodeId;
    }

    /**
     * 
     * @param nodes
     * @param maxNodeId
     * @return maximum id 
     */
    private int getMaxId( Node[] nodes, int maxNodeId ) {
        for ( int i = 0; i < nodes.length; i++ ) {
            Node[] brancheNodes = nodes[i].getNodes();
            if ( nodes[i].getId() > maxNodeId ) {
                maxNodeId = nodes[i].getId();

            }
            maxNodeId = getMaxId( brancheNodes, maxNodeId );
        }
        return maxNodeId;
    }

    /**
     * 
     * @param parent
     */
    public void setParent( Node parent ) {
        this.parent = parent;
    }

    /**
     * 
     * @return parent node
     */
    public Node getParent() {
        return parent;
    }

    /**
     * The id of the selected node.
     * 
     * @return id
     */
    public int getId() {
        return id;
    }

    /**
     * The title of the selected node.
     * 
     * @return title
     */
    public String getTitle() {
        return title;
    }

    /**
     * The status of the node (collapsed or not (expanded)).
     * 
     * @return true if node is collapsed
     */
    public boolean isCollapsed() {
        return collapsed;
    }

    /**
     * The selectable status of the node.
     * 
     * @return true if node is selectable
     */
    public boolean isSelectable() {
        return selectable;
    }

    /**
     * @param nodes
     */
    public void setNodes( Node[] nodes ) {
        if ( nodes == null ) {
            nodes = new Node[0];
        }
        this.nodes = nodes;
    }

    /**
     * @param selectable
     */
    public void setSelectable( boolean selectable ) {
        this.selectable = selectable;
    }

    /**
     * @param collapsed
     */
    public void setCollapsed( boolean collapsed ) {
        this.collapsed = collapsed;
    }

    /**
     * @param id
     */
    public void setId( int id ) {
        this.id = id;
    }

    /**
     * 
     * @param title
     * 
     * @throws ContextException
     */
    public void setTitle( String title )
                            throws ContextException {
        if ( title == null ) {
            throw new ContextException( "title isn't allowed to be null" );
        }
        this.title = title;
    }

    /**
     * 
     * @return flat tree as node matrix
     */
    public Node[][] getFlatTree() {
        tree = new ArrayList<Node[]>();
        tree = getBranches( nodes );
        Node[][] flatTree = new Node[tree.size()][];
        return tree.toArray( flatTree );
    }

    /**
     * 
     * @param nodes
     * @return tree branches
     */
    private List<Node[]> getBranches( Node[] nodes ) {
        for ( int i = 0; i < nodes.length; i++ ) {
            Node[] branchNodes = nodes[i].getNodes();
            if ( branchNodes.length == 0 ) {
                List<Node> treeRowList = new ArrayList<Node>( 50 );
                Node tmpNode = nodes[i];
                while ( tmpNode != null ) {
                    treeRowList.add( tmpNode );
                    tmpNode = tmpNode.getParent();
                }
                Collections.reverse( treeRowList );
                Node[] treeRow = new Node[treeRowList.size()];
                tree.add( treeRowList.toArray( treeRow ) );
            } else {
                getBranches( branchNodes );
            }
        }
        return tree;

    }

    /**
     * moves a node within the tree up or down
     * 
     * @param nodeId
     * @param up
     */
    public void moveNode( int nodeId, Boolean up ) {

        for ( int i = 0; i < nodes.length; i++ ) {
            if ( nodes[i].getId() == nodeId ) {
                Node source = null;
                Node target = null;
                if ( up ) {
                    source = nodes[i];
                    target = nodes[i - 1];
                    nodes[i] = target;
                    nodes[i - 1] = source;
                    return;
                }
                source = nodes[i];
                target = nodes[i + 1];
                nodes[i] = target;
                nodes[i + 1] = source;
                return;

            }
            nodes[i].moveNode( nodeId, up );
        }

    }

}
